/** @file		NetF_BytesPerSecond.h
 *  @brief	Type safe data rate calculations
 *  @author	netfinity
 */
#pragma once

/**
 *  @brief	Container for data rates
 */
class CBytesPerSecond
{
public:
	CBytesPerSecond(const sint32 dwBytesPerSecond = 0);
	CBytesPerSecond(const CBytesPerSecond& bytespersecond);
	// Conversions
	sint32 GetValue() const;
	// Assignments
	CBytesPerSecond operator += (const CBytesPerSecond& bytespersecond);
	CBytesPerSecond operator -= (const CBytesPerSecond& bytespersecond);
	// Comparisations
	bool operator == (const CBytesPerSecond& bytespersecond) const;
	bool operator != (const CBytesPerSecond& bytespersecond) const;
	bool operator >= (const CBytesPerSecond& bytespersecond) const;
	bool operator <= (const CBytesPerSecond& bytespersecond) const;
	bool operator > (const CBytesPerSecond& bytespersecond) const;
	bool operator < (const CBytesPerSecond& bytespersecond) const;
	// Arithmetics
	CBytesPerSecond operator - (void) const;
	CBytesPerSecond operator + (const CBytesPerSecond& bytespersecond) const;
	CBytesPerSecond operator - (const CBytesPerSecond& bytespersecond) const;
	CBytesPerSecond operator * (const float& multiplier) const;
	CBytesPerSecond operator / (const float& divisor) const;
	CBytesPerSecond operator % (const CBytesPerSecond& bytespersecond) const;
private:
	sint32	m_dwBytesPerSecond;
};

//
//	inline operations
//

#include "NetF/NetF_MilliSeconds.h"
#include "NetF/NetF_Bytes.h"

inline
CBytesPerSecond::CBytesPerSecond(const sint32 dwBytesPerSecond)
{
	m_dwBytesPerSecond = dwBytesPerSecond;
}

inline
CBytesPerSecond::CBytesPerSecond(const CBytesPerSecond& bytespersecond)
{
	m_dwBytesPerSecond = bytespersecond.m_dwBytesPerSecond;
}

inline
sint32
CBytesPerSecond::GetValue() const
{
	return m_dwBytesPerSecond;
}

inline
bool
CBytesPerSecond::operator == (const CBytesPerSecond& bytespersecond) const
{
	return m_dwBytesPerSecond == bytespersecond.m_dwBytesPerSecond;
}

inline
bool
CBytesPerSecond::operator != (const CBytesPerSecond& bytespersecond) const
{
	return m_dwBytesPerSecond != bytespersecond.m_dwBytesPerSecond;
}

inline
bool
CBytesPerSecond::operator >= (const CBytesPerSecond& bytespersecond) const
{
	return m_dwBytesPerSecond >= bytespersecond.m_dwBytesPerSecond;
}

inline
bool
CBytesPerSecond::operator <= (const CBytesPerSecond& bytespersecond) const
{
	return m_dwBytesPerSecond <= bytespersecond.m_dwBytesPerSecond;
}

inline
bool
CBytesPerSecond::operator > (const CBytesPerSecond& bytespersecond) const
{
	return m_dwBytesPerSecond > bytespersecond.m_dwBytesPerSecond;
}

inline
bool
CBytesPerSecond::operator < (const CBytesPerSecond& bytespersecond) const
{
	return m_dwBytesPerSecond < bytespersecond.m_dwBytesPerSecond;
}

inline
CBytesPerSecond
CBytesPerSecond::operator += (const CBytesPerSecond& bytespersecond)
{
	return m_dwBytesPerSecond += bytespersecond.m_dwBytesPerSecond;
}

inline
CBytesPerSecond
CBytesPerSecond::operator -= (const CBytesPerSecond& bytespersecond)
{
	return m_dwBytesPerSecond -= bytespersecond.m_dwBytesPerSecond;
}

inline
CBytesPerSecond
CBytesPerSecond::operator - (void) const
{
	return CBytesPerSecond(-m_dwBytesPerSecond);
}

inline
CBytesPerSecond
CBytesPerSecond::operator + (const CBytesPerSecond& bytespersecond) const
{
	return CBytesPerSecond(m_dwBytesPerSecond + bytespersecond.m_dwBytesPerSecond);
}

inline
CBytesPerSecond
CBytesPerSecond::operator - (const CBytesPerSecond& bytespersecond) const
{
	return CBytesPerSecond(m_dwBytesPerSecond - bytespersecond.m_dwBytesPerSecond);
}

inline
CBytesPerSecond
CBytesPerSecond::operator * (const float& multiplier) const
{
	return CBytesPerSecond(m_dwBytesPerSecond * multiplier);
}

inline
CBytesPerSecond
CBytesPerSecond::operator / (const float& divisor) const
{
	return CBytesPerSecond(m_dwBytesPerSecond / divisor);
}

inline
CBytesPerSecond
CBytesPerSecond::operator % (const CBytesPerSecond& bytespersecond) const
{
	return CBytesPerSecond(m_dwBytesPerSecond % bytespersecond.m_dwBytesPerSecond);
}

//
//	Special arithmetics
//

inline
CBytesPerSecond
operator / (const CBytes& bytes, const CMilliSeconds& milliseconds)
{
	return CBytesPerSecond((bytes.GetValue() * 1000) / (sint64) milliseconds.GetValue());
}

inline
CMilliSeconds
operator / (const CBytes& bytes, const CBytesPerSecond& bytespersecond)
{
	return CMilliSeconds(((sint64) bytes.GetValue() * 1000) / bytespersecond.GetValue());
}

inline
float
operator / (const CBytesPerSecond& bytespersecond1, const CBytesPerSecond& bytespersecond2)
{
	return ((float) bytespersecond1.GetValue() / (float) bytespersecond2.GetValue());
}

inline
CBytes
operator * (const CBytesPerSecond& bytespersecond, const CMilliSeconds& milliseconds)
{
	return CBytes((sint64) bytespersecond.GetValue() * (sint64) milliseconds.GetValue() / 1000);
}

inline
CBytes
operator * (const CMilliSeconds& milliseconds, const CBytesPerSecond& bytespersecond)
{
	return CBytes((sint64) milliseconds.GetValue() * (sint64) bytespersecond.GetValue() / 1000);
}

inline
CBytesPerSecond
operator * (const float& multiplier, const CBytesPerSecond& bytespersecond)
{
	return CBytesPerSecond(multiplier * bytespersecond.GetValue());
}

//
//	
//

#include "OtherFunctions.h"

inline
CString
CastItoXBytes(const CBytesPerSecond& bytespersecond, uint32 decimal = 2)
{
	if (bytespersecond >= 0)
		return ::CastItoXBytes((uint32) bytespersecond.GetValue(), false, true, decimal);
	else
		return _T("-") + ::CastItoXBytes((uint32) (-bytespersecond.GetValue()), false, true, decimal);
}
